import datetime
import time
from data_mod import DataMod
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
import cdo as cdop
from scipy.io.netcdf import *
import copy
import numpy
import shutil
from common import *
#from nco import Nco


class DataEC23(DataMod):
     """ classe of data for the b0cm and b0gq experiments
     """
     def __init__(self, dataname, varname):
          DataMod.__init__(self, dataname, varname)
          self._nmember=10
          self._freq=datetime.timedelta(0, 0, 0, 0, 0, 6)
          
     def set_inputdirAuto(self):
          """defining the path to look for the model daily data"""
          dataname=self.get_dataname()
          varname=self.get_varname()
          
          suff="_6hourly"
          inputdir = "/esnas/exp/ecearth/"+dataname+"/daily_grb/" + varname + suff +"/"
          
          if (not os.path.isdir(inputdir)):
               print "This directory does not exist:"
               print inputdir
               #sys.exit(0)

          self.set_inputdir(inputdir)


     def format_files_nc(self):
          """transform the grb file 
          into ncdf file readable for cdo"""
          
          #cdo=Cdo()

          cdo=cdop.Cdo()
          cdo.tempfile.tempdir = os.environ['TMPDIR']
                
          #get the needed variables from the object
          filelist=self.get_filelist()
          sdates=self.get_sdates()
          dataname=self.get_dataname()
          varname=self.get_varname()
          nmember=self.get_nmember()
          filetype=self.get_filetype()

        #in ECMWF S4 the leadtime is 7months
        #hard coded
        # hard coded find a better solution
        #select the step
          if (dataname in ["b0cm", "b0gq"]):
               nbmon=relativedelta(months=4, days=-1)
          else:
               nbmon=relativedelta(months=+7, days=-1)
               
          if varname in ["lsp", "cp"]:
               self.set_freq(datetime.timedelta(0, 0, 0, 0, 0, 24))
               
          freq=self.get_freq()
        
          for ifile in range(len(filelist)):
               f=filelist[ifile]
               d1=sdates[ifile]
                              
            #check if the output file is already here
               out="format_"+varname+"_"+str(d1).replace("-","")+".nc"
               if len(glob.glob(out))==0:
                    
                    flocal=f.split("/")[-1]
                #copy the file in the scratch (faster)
                    if len(glob.glob(flocal))==0:
                         shutil.copy(f,".")
                    f=flocal
                  
                #calculate the last date of the file
                    d2=d1+nbmon
                
                #calculate the number of timestep
                    if freq.seconds != 0:
                         nbstep=((d2-d1).days + 1)*86400/freq.seconds
                    else:
                         nbstep=((d2-d1).days + 1) + 1

                #list to store the members
                    lstmemb=[]
                #loop over the member
                    for imemb in range(nmember):
                         print(imemb)
                         # hard coded find a better solution
                    #select the step
                         if (dataname in ["b0cm", "b0gq"]):
                              step=[i * nmember +imemb for i in range(nbstep)]
                         else:
                              step=list(numpy.array(range(nbstep))+nbstep*imemb)
                    #convert step in string
                         step=str(step).strip('[]').replace(" ","")
                    #select one member
                         fmemb=f+str(imemb)
                         print(fmemb)
                         cdo.seltimestep(step,  
                                         input=f, output=fmemb, options="-r")
                         lstmemb.append(fmemb)

                       #print(lstmemb)
                #cat the ensemble members in a a new dimension
                    lstmembstr=str(lstmemb).strip('[]').replace(", "," ").replace("'","")

                    dictvar={"tas":"T2M","lsp":"LSP","cp":"CP","psl":"MSL",
                             "uas":"U10M","vas":"V10M",
                             "mrso1":"SWVL1","mrso2":"SWVL2",
                             "mrso3":"SWVL3","mrso4":"SWVL4"}
                    oldname=dictvar.get(varname)
                    if oldname == None:
                         print("the variable "+varname+"is no available in the format_file function")
                         sys.exit(0)
                    cdo.chvar(oldname,varname, 
                              input=cdo.merge(input=lstmembstr,output=out+"tmp",
                                              options="-r", 
                                              force =  False),
                              output=out, options="-r", 
                              force =  False)
                #remove the file per member to save space
                    os.remove(out+"tmp")
                    for filetmp in lstmemb:
                         os.remove(filetmp)
                #remove the copy of the original file to save space
                    os.remove(flocal)

               print(out)
               filelist[ifile]=out
          self.set_filelist=list(filelist)



     def ready(self, date1, date2):
          """ make the Data object ready to calculate statistic
          """
          #cdo=Cdo()
          
          cdo=cdop.Cdo()
          cdo.tempfile.tempdir = os.environ['TMPDIR']
          dataname=self.get_dataname()
          varname=self.get_varname()
        #test if the file are already in the workdir
          self.set_inputdir("")
          test=self.set_files(date1,date2)
          self.set_inputdirAuto()
          self.set_outputdirAuto() 
          self.see()
          print(test)
          if test == 1:
               if varname == "tasmin":
                    self.set_varname("tas")
                    self.set_inputdirAuto()
                    self.set_outputdirAuto() #look for the output directory
                    self.set_files(date1,date2) #look for the files between date1 and date2 
                    self.format_files_nc() #transform grib in nc files cdo compatible
                    self.cdo_statistic("daymin","tasmin")
                    self.see()
               if varname == "tas":
                    self.set_varname("tas")
                    self.set_inputdirAuto()
                    self.set_outputdirAuto() #look for the output directory
                    self.set_files(date1,date2) #look for the files between date1 and date2 
                    self.format_files_nc() #transform grib in nc files cdo compatible
                    self.cdo_statistic("daymean","tas")
                    self.see()
               elif varname == "tasmax":
                    self.set_varname("tas")
                    self.set_inputdirAuto()
                    self.set_outputdirAuto() #look for the output directory
                    self.set_files(date1,date2) #look for the files between date1 and date2 
                    self.format_files_nc() #transform grib in nc files cdo compatible
                    self.cdo_statistic("daymax",varname)
               elif varname == "psl18h":
                    self.set_varname("psl")
                    self.set_inputdirAuto()
                    self.set_outputdirAuto() #look for the output directory
                    self.set_files(date1,date2) #look for the files between date1 and date2 
                    self.format_files_nc() #transform grib in nc files cdo compatible
                    self.cdo_statistic("selhour,18:00:00",varname)
               elif varname == "prlr":
                    auxlsp=DataEC23(dataname,"lsp")
                    auxlsp.set_inputdirAuto()
                    auxlsp.set_files(date1,date2) #look for the files between date1 and date2 
                    auxlsp.format_files_nc() #transform grib in nc files cdo compatible
                    
                    auxcp=DataEC23(dataname,"cp")
                    auxcp.set_inputdirAuto()
                    auxcp.set_files(date1,date2) #look for the files between date1 and date2 
                    auxcp.format_files_nc() #transform grib in nc files cdo compatible
                    
                    aux=weighted_sum((auxlsp, auxcp), (1,1), varname)
                    aux.set_outputdirAuto() #look for the output directory
                    self.copy(aux)
               elif varname == "sfcwind":
                    
                    auxuas=DataEC23(dataname,"uas")
                    auxuas.set_inputdirAuto()
                    auxuas.set_files(date1,date2) #look for the files between date1 and date2 
                    auxuas.see()
                    auxuas.format_files_nc()
                    auxuas.see()
                    
                    auxvas=DataEC23(dataname,"vas")
                    auxvas.set_inputdirAuto()
                    auxvas.set_files(date1,date2) #look for the files between date1 and date2 
                    auxvas.see()
                    auxvas.format_files_nc()
                    auxvas.see()
                    
                    self.copy(module(auxuas,auxvas, "sfcwind"))
                    self.see()
                    self.set_outputdirAuto() #look for the output directory
               elif varname == "mrso":
                    
                    aux1=DataEC23(dataname,"mrso1")
                    aux1.set_inputdirAuto()
                    aux1.set_files(date1,date2) #look for the files between date1 and date2 
                    aux1.see()
                    aux1.format_files_nc()
                    aux1.see()
                    
                    aux2=DataEC23(dataname,"mrso2")
                    aux2.set_inputdirAuto()
                    aux2.set_files(date1,date2) #look for the files between date1 and date2 
                    aux2.see()
                    aux2.format_files_nc()
                    aux2.see()
                
                    aux3=DataEC23(dataname,"mrso3")
                    aux3.set_inputdirAuto()
                    aux3.set_files(date1,date2) #look for the files between date1 and date2 
                    aux3.see()
                    aux3.format_files_nc()
                    aux3.see()
                    
                    aux4=DataEC23(dataname,"mrso4")
                    aux4.set_inputdirAuto()
                    aux4.set_files(date1,date2) #look for the files between date1 and date2 
                    aux4.see()
                    aux4.format_files_nc()
                    aux4.see()
             
                    aux=weighted_sum((aux1,aux2,aux3,aux4), 
                                     (7./255,21./255,72./255,155./255), varname)
                    self.copy(aux)
                    self.see()
                    self.set_outputdirAuto() #look for the output directory
                    
               else:
                    self.set_varname(varname)
                    self.set_inputdirAuto()
                    self.set_outputdirAuto() #look for the output directory
                    self.set_files(date1,date2) #look for the files between date1 and date2 
                    self.format_files_nc() #transform grib in nc files cdo compatible
                    self.see()
